#!/bin/bash
#
# Little bit advanced script for deploying Luci (prepare application
# + database and start serving it).
#
# Return codes:
#    0  All should be OK.
#
#    1  Bad use of arguments.
#    2  Configuration file not specified.
#    3  Host address number not specified.
#    4  Port number number not specified.
#
#   10  Start of tg2env failed.
#   11  Execution of 'python setup.py' failed.
#   12  Execution of 'paster setup-app' failed.
#   13  Execution of 'paster serve' failed.

# -----------------------------------------------------------------------------
# Defaults and flags.

def_tg2env_dir=~/tg2env
def_config_file=development.ini

flag_runonly=0
flag_background=0
flag_devel=0
flag_use_tg2env=0
flag_change_cfg=0

if [ "$BASH_SOURCE" == "$0" ]
then
    cmd_exit="exit"
    self=`basename $0`
else
    cmd_exit="return"
    self=`basename $BASH_SOURCE`
fi

# -----------------------------------------------------------------------------
# Helper functions.

usage() {
  echo "Brief usage (use -h or --help for more information):"
  echo "[source] ./$self [-r|-b|-d|-t [<dir>]|-c <file>|-a <host>|-p <port>]"
}

help() {
  echo "Usage:"
  echo "  <run command> -h|--help"
  echo "  <run command> [<deploy options>]"
  echo
  echo "<run command> can be (\`source-form' recommended for '-b -t' combination)"
  echo "  ./$self"
  echo "  [.|source] $self"
  echo
  echo "<deploy options> are following:"
  echo "  -r, --run-only"
  echo "  -b, --background"
  echo "  -d, --devel"
  echo "  -t, --tg2env [<tg2env directory>]"
  echo "  -c, --config <configuration file for paster>"
  echo "  -a, --address <server's host adress>"
  echo "  -p, --port <server's port number>"
  echo
  echo "-> \`background' ... server will be started in daemon (background) mode"
  echo "   * to stop the server, use 'paster --stop-daemon' (given a PID file,"
  echo "     default is paster.pid)"
  echo "   * if used with -t option, \`source $self' form strongly recommended"
  echo "-> \`devel' ... server will be restarted if any of the files"
  echo "   of the application being served is changed (suitable for developing)"
  echo "-> \`run-only ' ... initialization routines (e.g. checking dependencies"
  echo "   and preparing database) required to take place on the first"
  echo "   application run are skipped (i.e. this option is suitable for every"
  echo "   other run of the application than the first one)"
  echo "-> when --tg2env option used and no directory given, \`$def_tg2env_dir'"
  echo "   is used as default"
  echo "-> when --config option with its argument NOT specified,"\
           "\`$def_config_file'"
  echo "   is used as default"
}

at_exit() {
    unset tg2env_dir
    unset config_file
    unset host
    unset port
    unset paster_pre_options
    unset paster_post_options
}

# -----------------------------------------------------------------------------
# Body of the script.

# Parse command-line options and arguments.
while [ $# -gt 0 ]
do
	case $1 in
	    -h | --help )
	        shift
	        help
	        at_exit
	        $cmd_exit 0
	    ;; #--------
	    -r | --run-only )
	        flag_runonly=1
	        shift
	    ;; #--------
	    -b | --background )
	        flag_background=1
	        shift
	    ;; #--------
	    -d | --devel )
	        flag_devel=1
	        shift
	    ;; #--------
	    -t | --tg2env )
	        flag_use_tg2env=1
		    shift
	        if [ $# -ne 0 -a "${1:0:1}" != "-" -a "${1:0:2}" != "-" ]
	        then
	            tg2env_dir=$1
	            shift
	        fi
	    ;; #--------
	    -c | --config )
	        shift
	        if [ $# -gt 0 ]
	        then
	            config_file=$1
	            shift
	        else
	            echo "ERROR: Configuration file not specified."  > /dev/stderr
	            usage
	            at_exit
	            $cmd_exit 2
	        fi
	    ;; #--------
	    -a | --address )
	        shift
	        if [ $# -gt 0 ]
	        then
	            flag_change_cfg=1
	            host=$1
	            shift
	        else
	            echo "ERROR: Host address number not specified."  > /dev/stderr
	            usage
	            at_exit
	            $cmd_exit 3
	        fi
	    ;; #--------
	    -p | --port )
	        shift
	        if [ $# -gt 0 ]
	        then
	            flag_change_cfg=1
	            port=$1
	            shift
	        else
	            echo "ERROR: Port number number not specified."
	            usage
	            at_exit
	            $cmd_exit 4
	        fi
	    ;; #--------
	    * )
	        echo "ERROR: Bad use of arguments."
	        usage
	        at_exit
	        $cmd_exit 1
    esac
done


# Realize, what configuration file will be used and backup the old one
# if it will be changed.
config_file=${config_file:=$def_config_file}
if (($flag_change_cfg))
then
    cp $config_file $config_file.$(date +%y%m%d%H%M)
fi


# Start tg2env if requested.
if (($flag_use_tg2env))
then
    tg2env_dir=${tg2env_dir:=$def_tg2env_dir}
    echo "INFO: Starting tg2env environment from \`$tg2env_dir' ..."
    source $tg2env_dir/bin/activate
    exit_failure=$?
    if ((! $exit_failure))
    then
        echo "INFO: Start of tg2env was succesful."
    else
        echo "ERROR: Start of tg2env failed (exit code $exit_failure)."\
              > /dev/stderr
        at_exit
        $cmd_exit 10
    fi
else
    echo "NOTE: To use tg2env environment, use '--tg2env <tg2env directory>" \
         "option or set def_tg2env_dir variable in the script."
fi


# Change the host address in the configuration file if requested.
if [ "${host:=host not set}" != "host not set" ]
then
    echo "INFO: Changing host address to \`$host' in the file \`$config_file'."
    sed -i "s/^\([ \t]*host[ \t]*=\)[^$]*$/\1 $host/1" $config_file
fi


# Change the port number in the configuration file if requested.
if [ "${port:=port not set}" != "port not set" ]
then
    echo "INFO: Changing port number to \`$port' in the file \`$config_file'."
    sed -i "s/^\([ \t]*port[ \t]*=\)[^$]*$/\1 $port/1" $config_file
fi


# Run the process of application initialization if not requested to skip it.
if ((! $flag_runonly))
then
    if ((! $flag_devel))
    then
        # Following will create 'egg' file under site-packages.
        python setup.py install
    else
        # Following will create 'egg-link' (poor man's symlink) file under
        # site-packages.
        python setup.py develop
    fi
    exit_failure=$?
    #XXX if (($exit_failure))
    if [ $exit_failure -gt 1 ]
    then
        echo "ERROR: Execution of 'python setup.py' failed"\
             "(exit code $exit_failure)." > /dev/stderr
        at_exit
        $cmd_exit 11
    fi

    paster setup-app $config_file
    exit_failure=$?
    #XXX if (($exit_failure))
    if [ $exit_failure -gt 1 ]
    then
        echo "ERROR: Execution of 'paster setup-app' failed"\
             "(exit code $exit_failure)." > /dev/stderr
        at_exit
        $cmd_exit 12
    fi
fi


# Background monitoring of changes in source files?
if (($flag_devel))
then
    paster_pre_options+=" --reload-interval=5"
fi

# Start server in daemon (background) mode?
if (($flag_background))
then
    paster_pre_options+=" --daemon"
    # paster_post_options+=" start" # Not necessary.
fi

# Run the server.
echo "Running 'paster_pre_options $config_file $paster_post_options'."
paster serve $paster_pre_options $config_file $paster_post_options
exit_failure=$?
if (($exit_failure))
then
    echo "ERROR: Execution of 'paster serve' failed"\
         "(exit code $exit_failure)." > /dev/stderr
    at_exit
    $cmd_exit 13
fi

# If server started in daemon (background) mode and using tg2env, run
# subshell offering this environment to eventually stop the server.
if (($flag_background))
then
	echo
	echo "-> Server is running in daemon mode now, you can use:"
	echo "    \$ paster serve $config_file stop/restart/status"
	echo "to make it stopped, restarted or to get its status, respectively."

    if (($flag_use_tg2env))
    then
        echo "-> To deactivate tg2env environment, use:"
        echo "    \$ deactivate"
    fi
elif (($flag_use_tg2env))
then
	deactivate
fi

at_exit
$cmd_exit 0

